-- *********************************************************************/ 
-- Copyright 2003 Actel Corporation.  All rights reserved.
-- IP Solutions Group
--  
-- File:  demo1553SX.vhd
--     
-- Description: 1553B BC Demo Design
--              Top Level of Design for SX Devices using PCI/1553B demo Board  
--
-- Rev: 1.0 01Oct02 IPB  : Initial Code  
-- Rev: 2.1 22Aug03 IPB  : Production 2.0
--
-- Notes:
-- This design inferfaces to the Synchronous SRAMS on the PCI demo board.
-- These RAMS run of the 33MHz PCI clock hence the need for the relative
-- complex RAM interface state machines. Ideally these RAM's should run of
-- the 16MHZ PCi clock - it would be a lot simpler
--
-- *********************************************************************/ 

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

library a54sxa;
use a54sxa.components.all;


entity DEMO1553W is
   port    ( -- The PCI Bus Signals
             CLKPCI     : in     std_logic;
             RSTN       : in     std_logic;
             ACK64N     : out    std_logic;                       
             AD         : inout  std_logic_vector(31 downto 0); 
             CBE        : in     std_logic_vector( 3 downto 0);  
             DEVSELN    : out    std_logic; 
             FRAMEN     : in     std_logic; 
             GNTN       : in     std_logic; 
             IDSEL      : in     std_logic; 
             INTAN      : out    std_logic;
             IRDYN      : in     std_logic; 
             PAR        : inout  std_logic; 
             PAR64      : inout  std_logic; 
             PERRN      : out    std_logic; 
             REQ64N     : inout  std_logic; 
             REQN       : out    std_logic; 
             SERRN      : out    std_logic; 
             STOPN      : out    std_logic; 
             TRDYN      : out    std_logic; 
             M66EN      : out    std_logic;
             
             --1553B Interface
             CLK1553    : in    std_logic;
             BUSAINEN   : out   std_logic;
             BUSAINP    : in    std_logic;
             BUSAINN    : in    std_logic;
             BUSBINEN   : out   std_logic;
             BUSBINP    : in    std_logic;
             BUSBINN    : in    std_logic;
             BUSAOUTIN  : out   std_logic;
             BUSAOUTP   : out   std_logic;
             BUSAOUTN   : out   std_logic;
             BUSBOUTIN  : out   std_logic;
             BUSBOUTP   : out   std_logic;
             BUSBOUTN   : out   std_logic;

             -- UART Interface
             RS232TXD   : out   std_logic;
             RS232RXD   : in    std_logic; 
             RS232RTS   : out   std_logic;
             RS232CTS   : in    std_logic;
             RS232DTR   : out   std_logic;
             RS232DCD   : in    std_logic;
             RS232DSR   : in    std_logic;
             RS232RI    : in    std_logic;
             
             -- SSRAM or SDRAM Interface
             MEMADDR    : out   std_logic_vector(17 downto 0);
             MEMBNKSEL  : out   std_logic_vector( 1 downto 0); 
             MEMDQM     : out   std_logic_vector( 7 downto 0);
             MEMCSN     : out   std_logic_vector( 4 downto 0);
             MEMWEN     : out   std_logic;
             MEMCASN    : out   std_logic;
             MEMRASN    : out   std_logic;
             MEMADSCN   : out   std_logic;
             MEMADVN    : out   std_logic;
             MEMCKE     : out   std_logic;
             MEMDATA    : inout std_logic_vector(63 downto 0); 
             MEMDP      : inout std_logic_vector( 7 downto 0); 
             
             -- ROBO clock control
             ROBOTEST   : out   std_logic;
             ROBOFS     : out   std_logic;
             
             -- I2C interface on SDRM
             I2CSCI     : out   std_logic;
             I2CSDA     : out   std_logic;
   
             -- LEDS
             LEDS       : out   std_logic_vector( 3 downto 0);
             SPARES     : out   std_logic_vector(10 downto 0)
           ); 
end DEMO1553W;   
      
   
architecture RTL of DEMO1553W is


component PCI1553BBC
   port    ( -- The PCI Bus Signals
             PCICLK     : in     std_logic;
             PCIRSTN    : in     std_logic;
             AD         : inout  std_logic_vector(31 downto 0);
             CBE        : in     std_logic_vector(3 downto 0); 
             DEVSELN    : out    std_logic;  
             FRAMEN     : in     std_logic;   
             IDSEL      : in     std_logic;   
             INTAN      : out    std_logic;
             IRDYN      : in     std_logic;   
             PAR        : inout  std_logic;  
             PERRN      : out    std_logic;  
             SERRN      : out    std_logic;  
             STOPN      : out    std_logic;  
             TRDYN      : out    std_logic;  
   
             -- 1553B Clock and Tranceiver
             CLK1553    : in  std_logic;
             BUSAINEN   : out std_logic;
             BUSAINP    : in  std_logic;
             BUSAINN    : in  std_logic;
             BUSBINEN   : out std_logic;
             BUSBINP    : in  std_logic;
             BUSBINN    : in  std_logic;
             BUSAOUTIN  : out std_logic;
             BUSAOUTP   : out std_logic;
             BUSAOUTN   : out std_logic;
             BUSBOUTIN  : out std_logic;
             BUSBOUTP   : out std_logic;
             BUSBOUTN   : out std_logic;
             

             -- SSRAM Interface
             RAMADDR    : out   std_logic_vector(14 downto 0);           
             RAMDATA    : inout std_logic_vector(31 downto 0);           
             RAMCSN     : out   std_logic;           
             RAMWRN     : out   std_logic_vector(3 downto 0);           
             RAMRDN     : out   std_logic;           
             RAMADSN    : out   std_logic;           
             RAMOEN     : out   std_logic;
                          
             -- LEDS Etc
             LEDS       : out std_logic_vector(3 downto 0);
             PROBE      : out std_logic_vector(3 downto 0)
           ); 
end component;   
      


-- Miscellanous
signal  CLK16       : std_logic;
signal  CLK33       : std_logic;
signal  CLKXX       : std_logic;
signal  CLKDIV      : std_logic_vector(1 downto 0) := "00";
signal  RESETn      : std_logic;
signal  RTADDR      : std_logic_vector(4 downto 0);           
signal  RTADDRP     : std_logic;

signal  XSPARE      : std_logic;
signal  INPUTSPCI   : std_logic;
signal  INPUTSRS232 : std_logic;

            
-- SSRAM Interface
signal  RAMADDR     : std_logic_vector(14 downto 0);           
signal  RAMCSN      : std_logic;           
signal  RAMRDN      : std_logic;           
signal  RAMWRN      : std_logic_vector(3 downto 0);           
signal  RAMADSN     : std_logic;           
signal  RAMOEN      : std_logic;          

signal  PROBE       : std_logic_vector( 3 downto 0);
signal  ZERO        : std_logic_vector(63 downto 0);
signal  Logic0      : std_logic;
signal  Logic1      : std_logic;

     
begin

ZERO     <= ( others => '0');
Logic0   <= '0';
Logic1   <= '1';

-------------------------------------------------------------------------------
-- Tie all unused outputs to Logic Z
-- All unused inputs drive some dummy logic to stop Designer optimizing the pins

-- PCI Bus is in 33 MHz 32 Bit mode
ACK64N    <= 'Z';                        
M66EN     <= '0';   

-- RS232 Interface
RS232DTR    <= RS232DSR;
RS232TXD    <= RS232RXD;
RS232RTS    <= RS232CTS;
INPUTSRS232 <= RS232CTS or RS232DCD or RS232DSR or RS232RI;

-- I2C Interface SDRAM
I2CSCI  <= 'Z'; 
I2CSDA  <= 'Z'; 
      
-- Puts some logic on usused pins to stop designer optimizing them
XSPARE  <= GNTN or RS232CTS or RS232DCD or RS232DSR or RS232RI; 

  
-------------------------------------------------------------------------------
-- Clock Buffering and ROBO Clock Control
-- 

ROBOTEST  <= '0';
ROBOFS    <= 'Z'; -- MID Point operation for 33MHz


UCLKH : HCLKBUF port map ( PAD => CLKPCI,  Y => CLK33);
UCLKA : CLKINT  port map ( A   => CLK1553, Y => CLK16);  

-- Note the PCI core uses a global for IRDY 
 

-------------------------------------------------------------------------------
-- The 1553B Core Connecetd to the PCI core
--  

U1553B : PCI1553BBC 
   port map( PCICLK    => CLK33,      
             PCIRSTN   => RSTn,        
             AD        => AD,      
             CBE       => CBE,     
             DEVSELN   => DEVSELN, 
             FRAMEN    => FRAMEN,  
             IDSEL     => IDSEL,   
             INTAN     => INTAN,   
             IRDYN     => IRDYN,   
             PAR       => PAR,     
             PERRN     => PERRN,   
             SERRN     => SERRN,   
             STOPN     => STOPN,   
             TRDYN     => TRDYN,   
            
             CLK1553   => CLK16,     
             BUSAINEN  => BUSAINEN,    
             BUSAINP   => BUSAINP,     
             BUSAINN   => BUSAINN,     
             BUSBINEN  => BUSBINEN,    
             BUSBINP   => BUSBINP,     
             BUSBINN   => BUSBINN,     
             BUSAOUTIN => BUSAOUTIN,   
             BUSAOUTP  => BUSAOUTP,    
             BUSAOUTN  => BUSAOUTN,    
             BUSBOUTIN => BUSBOUTIN,   
             BUSBOUTP  => BUSBOUTP,    
             BUSBOUTN  => BUSBOUTN,    
             
             RAMADDR   => RAMADDR,            
             RAMDATA   => MEMDATA(31 downto 0),            
             RAMCSN    => RAMCSN,      
             RAMWRN    => RAMWRN,      
             RAMRDN    => RAMRDN,      
             RAMADSN   => RAMADSN,     
             RAMOEN    => RAMOEN,      
                                                                         
             LEDS      => LEDS,        
             PROBE     => PROBE      
           ); 
 
SPARES <= "000000" & XSPARE & PROBE;
     
-- Route Signals to the SSRAM on the PCI demoboard
-- 

-- Make sure that RAM parity inputs dont float during a write
MEMDP(7) <= '1' when RAMWRN(3)='0' else 'Z';
MEMDP(6) <= '1' when RAMWRN(2)='0' else 'Z';
MEMDP(5) <= '1' when RAMWRN(1)='0' else 'Z';
MEMDP(4) <= '1' when RAMWRN(0)='0' else 'Z';
MEMDP(3) <= '1' when RAMWRN(3)='0' else 'Z';
MEMDP(2) <= '1' when RAMWRN(2)='0' else 'Z';
MEMDP(1) <= '1' when RAMWRN(1)='0' else 'Z';
MEMDP(0) <= '1' when RAMWRN(0)='0' else 'Z';
  
MEMADDR   <= ZERO(17 downto 17) & RAMADDR & "00";
MEMCSN    <= RAMCSN & "1111";
MEMDQM    <= "1111" & RAMWRN;
MEMADSCN  <= RAMADSN;
MEMADVN   <= '1';
MEMBNKSEL <= "00";
MEMWEN    <= '1';
MEMCASN   <= '1';
MEMRASN   <= RAMOEN;
MEMCKE    <= '0';
 
end RTL;




