-- *********************************************************************/ 
-- Copyright 2003 Actel Corporation.  All rights reserved.
-- IP Solutions Group
--  
-- File:  pci1553bbc.vhd
--     
-- Description: 1553B BC Demo Design
--              PCI target to 1553B BC   
--
-- Rev: 1.0 01Oct02 IPB  : Initial Code  
-- Rev: 2.1 22Aug03 IPB  : Production 2.0
--
-- Notes:
--
-- *********************************************************************/ 

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;
Library Core1553BBC;

entity PCI1553BBC is
   port    ( -- The PCI Bus Signals
             PCICLK     : in     std_logic;
             PCIRSTN    : in     std_logic;
             AD         : inout  std_logic_vector(31 downto 0);
             CBE        : in     std_logic_vector(3 downto 0); 
             DEVSELN    : out    std_logic;  
             FRAMEN     : in     std_logic;   
             IDSEL      : in     std_logic;   
             INTAN      : out    std_logic;
             IRDYN      : in     std_logic;   
             PAR        : inout  std_logic;  
             PERRN      : out    std_logic;  
             SERRN      : out    std_logic;  
             STOPN      : out    std_logic;  
             TRDYN      : out    std_logic;  
   
             -- 1553B Clock and Tranceiver
             CLK1553    : in  std_logic;
             BUSAINEN   : out std_logic;
             BUSAINP    : in  std_logic;
             BUSAINN    : in  std_logic;
             BUSBINEN   : out std_logic;
             BUSBINP    : in  std_logic;
             BUSBINN    : in  std_logic;
             BUSAOUTIN  : out std_logic;
             BUSAOUTP   : out std_logic;
             BUSAOUTN   : out std_logic;
             BUSBOUTIN  : out std_logic;
             BUSBOUTP   : out std_logic;
             BUSBOUTN   : out std_logic;

             -- SSRAM Interface
             RAMADDR    : out   std_logic_vector(14 downto 0);           
             RAMDATA    : inout std_logic_vector(31 downto 0);           
             RAMCSN     : out   std_logic;           
             RAMWRN     : out   std_logic_vector(3 downto 0);           
             RAMRDN     : out   std_logic;           
             RAMADSN    : out   std_logic;           
             RAMOEN     : out   std_logic;
                          
             -- LEDS Etc
             LEDS       : out std_logic_vector(3 downto 0);
             PROBE      : out std_logic_vector(3 downto 0)
           ); 
end PCI1553BBC;   
      
   
architecture RTL of PCI1553BBC is


component PCITARG32  
   generic (MHZ_66          : INTEGER RANGE 0 TO 1  := 1;
            MADDR_WIDTH     : INTEGER RANGE 4 TO 31 := 18;
            BAR1_ENABLE     : INTEGER RANGE 0 TO 1  := 1;
            BAR1_IO_MEMORY  : INTEGER RANGE 0 TO 1  := 0; 
            BAR1_ADDR_WIDTH : INTEGER RANGE 2 TO 31 := 8; 
            BAR1_PREFETCH   : INTEGER RANGE 0 TO 1  := 0; 
            HOT_SWAP_ENABLE : INTEGER RANGE 0 TO 1  := 0); 
   port( 
        CLK           : in    std_logic;   
        RST           : in    std_logic;   
        AD            : inout std_logic_vector(31 downto 0); 
        CBE           : in    std_logic_vector(3 downto 0);  
        DEVSELN       : out   std_logic; 
        FRAMEN        : in    std_logic; 
        IDSEL         : in    std_logic; 
        INTAN         : out   std_logic;
        IRDYN         : in    std_logic; 
        PAR           : inout std_logic; 
        PERRN         : out   std_logic; 
        SERRN         : out   std_logic; 
        STOPN         : out   std_logic; 
        TRDYN         : out   std_logic; 
        BAR0_MEM_CYC  : out   std_logic;
        BAR1_CYC      : out   std_logic; 
        BE_GNT        : out   std_logic; 
        BE_REQ        : in    std_logic; 
        BUSY          : in    std_logic; 
        CONFIG_CYC    : out   std_logic;
        DP_DONE       : out   std_logic;
        DP_START      : out   std_logic;
        ERROR         : in    std_logic;     
        EXT_INTN      : in    std_logic;
        MEM_ADD       : out   std_logic_vector(MADDR_WIDTH-3 downto 0);  
        MEM_DIN       : in    std_logic_vector(31 downto 0); 
        MEM_DOUT      : out   std_logic_vector(31 downto 0); 
        PIPE_FULL_CNT : in    std_logic_vector(2 downto 0);
        RD_BE_NOW     : out   std_logic;
        RD_BE_RDY     : in    std_logic;  
        RD_CYC        : out   std_logic;
        WR_BE_NOW     : out   std_logic_vector(3 downto 0);
        WR_BE_RDY     : in    std_logic;   
        WR_CYC        : out   std_logic
       );       
end component; 

component BC1553B 
  port ( CLK        : in  std_logic;
         RSTINn     : in  std_logic;
         
         -- 1553B Interface
         BUSAINEN   : out std_logic;
         BUSAINP    : in  std_logic;
         BUSAINN    : in  std_logic;
         BUSBINEN   : out std_logic;
         BUSBINP    : in  std_logic;
         BUSBINN    : in  std_logic;
         BUSAOUTIN  : out std_logic;
         BUSAOUTP   : out std_logic;
         BUSAOUTN   : out std_logic;
         BUSBOUTIN  : out std_logic;
         BUSBOUTP   : out std_logic;
         BUSBOUTN   : out std_logic;
         
         -- Control Signals - should be tied high or low
         CPUMEMEN   : in  std_logic;
         ASYNCIF    : in  std_logic;
                  
         -- Status Signals
         BUSY       : out std_logic;
         INTOUT     : out std_logic;
         MEMFAIL    : out std_logic;
         EXTFLAG    : in  std_logic;
        
         -- CPU Interface
         CPUWRn     : in  std_logic_vector(1 downto 0);
         CPURDn     : in  std_logic;
         CPUCSn     : in  std_logic;
         CPUMEM     : in  std_logic;
         CPUWAITn   : out std_logic;
         CPUDEN     : out std_logic;
         CPUADDR    : in  std_logic_vector(15 downto 0);
         CPUDOUT    : out std_logic_vector(15 downto 0);
         CPUDIN     : in  std_logic_vector(15 downto 0);
                  
         -- Backend Memory interface
         MEMREQn    : out std_logic;
         MEMGNTn    : in  std_logic;
         MEMWRn     : out std_logic_vector(1 downto 0);
         MEMRDn     : out std_logic;
         MEMCSn     : out std_logic;
         MEMWAITn   : in  std_logic;
         MEMCEN     : out std_logic;
         MEMDEN     : out std_logic;
         MEMADDR    : out std_logic_vector(15 downto 0);
         MEMDOUT    : out std_logic_vector(15 downto 0);
         MEMDIN     : in  std_logic_vector(15 downto 0)
        );
end component;



-- PCI Backend Signals

signal  BAR0_MEM_CYC   : std_logic;
signal  BAR1_CYC       : std_logic; 
signal  DP_DONE        : std_logic;
signal  DP_START       : std_logic;
signal  ERROR          : std_logic;     
signal  EXT_INTN       : std_logic;
signal  PCIADDR        : std_logic_vector(17 downto 2);  
signal  PCIDIN         : std_logic_vector(31 downto 0); 
signal  PCIDOUT        : std_logic_vector(31 downto 0); 
signal  PIPE_FULL_CNT  : std_logic_vector(2 downto 0);
signal  RD_BE_NOW      : std_logic;
signal  RD_BE_RDY      : std_logic;  
signal  RD_CYC         : std_logic;
signal  WR_BE_NOW      : std_logic_vector(3 downto 0);
signal  WR_BE_RDY      : std_logic;   
signal  WR_CYC         : std_logic;

-- 1553B Signals      
signal  COREREQn      : std_logic;
signal  COREGNTn      : std_logic;
signal  COREWAITn     : std_logic;
signal  COREWRn       : std_logic_vector(1 downto 0);
signal  CORERDn       : std_logic;
signal  CORECSn       : std_logic;
signal  CORECEN       : std_logic;
signal  COREDEN       : std_logic;
signal  COREADDR      : std_logic_vector(15 downto 0);
signal  COREDOUT      : std_logic_vector(15 downto 0);

-- BC Signals
signal  CPUMEMEN      : std_logic;
signal  ASYNCIF       : std_logic;
signal  COREBUSY      : std_logic;
signal  COREINT       : std_logic;
signal  COREMFAIL     : std_logic;
signal  EXTFLAG       : std_logic;
signal  CPUWRn        : std_logic_vector(1 downto 0);
signal  CPURDn        : std_logic;
signal  CPUCSn        : std_logic;
signal  CPUMEM        : std_logic;
signal  CPUWAITn      : std_logic;
signal  CPUDEN        : std_logic;
signal  CPUADDR       : std_logic_vector(15 downto 0);
signal  CPUDOUT       : std_logic_vector(15 downto 0);
    
signal  CPUWRITE      : std_logic_vector(1 downto 0);
signal  WNOW          : std_logic_vector(1 downto 0);
signal  PCIREGACTIVE  : std_logic;
signal  WDELAY        : std_logic_vector(3 downto 0);
signal  CPURADDR      : std_logic_vector(2 downto 0);
signal  CPUDIN        : std_logic_vector(15 downto 0);
    
-- Clock Domain Synchronisation
signal  CORERD33      : std_logic;
signal  COREWR33      : std_logic;
signal  COREWAIT33    : std_logic;
signal  COREGNT33     : std_logic;
signal  COREREQ33     : std_logic;
    
-- Memory Interface State Machine
signal  PCIBUSY       : std_logic;
signal  PCIBUSY2      : std_logic;
signal  PCIRAMACC     : std_logic;
signal  PCIREGACC     : std_logic;
signal  PCIACTIVE     : std_logic;
signal  PCILIMIT      : std_logic;
signal  PCIACCCNT     : std_logic_vector(5 downto 0);
signal  RDDELAY       : std_logic_vector(1 downto 0);
  
-- SSRAM Memory Interface
signal  COREDIN       : std_logic_vector(15 downto 0);
signal  SSWRITEANY    : std_logic;
signal  SSREAD        : std_logic;
signal  SSWRITE       : std_logic_vector( 3 downto 0);
signal  SSDATA        : std_logic_vector(31 downto 0);
signal  SSDIN_NEG     : std_logic_vector(31 downto 0);
signal  SSOEN_NEG     : std_logic;
signal  SSADDR        : std_logic_vector(14 downto 0);

signal  PCCWORD       : std_logic_vector(15 downto 0);
signal  PCVWORD       : std_logic_vector(15 downto 0);

-- Miscellanous
signal  CLK33         : std_logic;
signal  CLK16         : std_logic;
signal  RESETn        : std_logic;
signal  LED1          : std_logic;
signal  LED2          : std_logic;
signal  LED3          : std_logic;
signal  LED4          : std_logic;

signal  Logic0        : std_logic;
signal  Logic1        : std_logic;
signal  ZERO          : std_logic_vector(31 downto 0);
    
function to_logic( x: BOOLEAN) return std_logic is
 variable y : std_logic;
 begin
  if x then y:='1';
       else y:='0';
  end if;
  return(y);
end to_logic;

     
begin

Logic0   <= '0';
Logic1   <= '1';
ZERO     <= ( others => '0');
  
-------------------------------------------------------------------------------
-- Clocks etc
--

CLK16 <= CLK1553;
CLK33 <= PCICLK;

RESETn <= PCIRSTn; 

LEDS(0) <= COREBUSY;
LEDS(1) <= COREINT;
LEDS(2) <= '0';
LEDS(3) <= COREMFAIL;

PROBE <= COREINT & COREBUSY & COREMFAIL & '0';

-------------------------------------------------------------------------------
-- The PCI Core
--  

UPCI : PCITARG32  
   generic map (MHZ_66          => 0, 
                MADDR_WIDTH     => 18,
                BAR1_ENABLE     => 0, 
                BAR1_IO_MEMORY  => 0, 
                BAR1_ADDR_WIDTH => 2, 
                BAR1_PREFETCH   => 0, 
                HOT_SWAP_ENABLE => 0  
               )
   port map  (  CLK           =>  CLK33,    
                RST           =>  RESETn,    
                AD            =>  AD,      
                CBE           =>  CBE,    
                DEVSELN       =>  DEVSELN, 
                FRAMEN        =>  FRAMEN, 
                IDSEL         =>  IDSEL,  
                INTAN         =>  INTAN,  
                IRDYN         =>  IRDYN,  
                PAR           =>  PAR,    
                PERRN         =>  PERRN,  
                SERRN         =>  SERRN,  
                STOPN         =>  STOPN,  
                TRDYN         =>  TRDYN, 
                BAR0_MEM_CYC  =>  BAR0_MEM_CYC, 
                BAR1_CYC      =>  BAR1_CYC,     
                BE_GNT        =>  Open,       
                BE_REQ        =>  Logic0,       
                BUSY          =>  PCIBUSY2,         
                CONFIG_CYC    =>  Open,   
                DP_DONE       =>  DP_DONE,      
                DP_START      =>  DP_START,     
                ERROR         =>  ERROR,        
                EXT_INTN      =>  EXT_INTN,     
                MEM_ADD       =>  PCIADDR,      
                MEM_DOUT      =>  PCIDOUT,     
                MEM_DIN       =>  PCIDIN,     
                PIPE_FULL_CNT =>  PIPE_FULL_CNT,
                RD_BE_NOW     =>  RD_BE_NOW,    
                RD_BE_RDY     =>  RD_BE_RDY,    
                RD_CYC        =>  RD_CYC,       
                WR_BE_NOW     =>  WR_BE_NOW,    
                WR_BE_RDY     =>  WR_BE_RDY,    
                WR_CYC        =>  WR_CYC       
              );


PCIRAMACC <= BAR0_MEM_CYC and not PCIADDR(17);
PCIREGACC <= BAR0_MEM_CYC and PCIADDR(17);

ERROR    <= '0';
EXT_INTN <= not COREINT; 

PIPE_FULL_CNT <= "000";   

PCIBUSY2 <= PCIBUSY and PCIRAMACC;

-------------------------------------------------------------------------------
-- The 1553B Core
--      


U1553B : BC1553B 
      port map  (  CLK        => CLK16,      
                   RSTINn     => RESETn,     
                   BUSAINEN   => BUSAINEN, 
                   BUSAINP    => BUSAINP, 
                   BUSAINN    => BUSAINN, 
                   BUSBINEN   => BUSBINEN,  
                   BUSBINP    => BUSBINP, 
                   BUSBINN    => BUSBINN, 
                   BUSAOUTIN  => BUSAOUTIN, 
                   BUSAOUTP   => BUSAOUTP, 
                   BUSAOUTN   => BUSAOUTN, 
                   BUSBOUTIN  => BUSBOUTIN, 
                   BUSBOUTP   => BUSBOUTP,  
                   BUSBOUTN   => BUSBOUTN,  
                   CPUMEMEN   => CPUMEMEN, 
                   ASYNCIF    => ASYNCIF,  
                   BUSY       => COREBUSY,     
                   EXTFLAG    => EXTFLAG,  
                   CPUWRn     => CPUWRn,   
                   CPURDn     => CPURDn,   
                   CPUCSn     => CPUCSn,   
                   CPUMEM     => CPUMEM,   
                   CPUWAITn   => CPUWAITn, 
                   CPUDEN     => CPUDEN,   
                   CPUADDR    => CPUADDR,  
                   CPUDOUT    => CPUDOUT,  
                   CPUDIN     => CPUDIN,   
                   INTOUT     => COREINT,
                   MEMFAIL    => COREMFAIL,
                   MEMREQn    => COREREQn,  
                   MEMGNTn    => COREGNTn,  
                   MEMWRn     => COREWRn,   
                   MEMRDn     => CORERDn,   
                   MEMCSn     => CORECSn,   
                   MEMWAITn   => COREWAITn, 
                   MEMCEN     => CORECEN,   
                   MEMDEN     => COREDEN,   
                   MEMADDR    => COREADDR,  
                   MEMDOUT    => COREDOUT,  
                   MEMDIN     => COREDIN   
                  );

CPUMEMEN  <= '0'; 
ASYNCIF   <= '1'; 
EXTFLAG   <= '0'; 
CPUMEM    <= '0';
CPUWRn(0) <= not CPUWRITE(0); 
CPUWRn(1) <= not CPUWRITE(1); 
CPURDn    <= '0'; 
CPUCSn    <= '0'; 
CPUADDR   <= ZERO(15 downto 3) & CPURADDR; 

-------------------------------------------------------------------------------
-- Synchronionise the 1553B BC Signals to the 33MHz PCI Clock
-- and vise versa

PMETA33:
process(CLK33)
begin
  if CLK33'event and CLK33='1' then
    COREREQ33  <= not COREREQn;
    CORERD33   <= not CORERDn;
    COREWR33   <= not COREWRn(0);
  end if;
end process;


PMETA16:
process(CLK16)
begin
  if CLK16'event and CLK16='1' then
    COREGNTn   <= not COREGNT33;
    COREWAITn  <= not COREWAIT33;
  end if;
end process;


-------------------------------------------------------------------------------
-- SSRAM Access Arbritartion etc
--  

-- PCI bus is allowed to do a burst that lasts 63 clocks at 33MHz 
-- allowing for max additional 20 clocks per 1553B access means the
-- 1553B is guarenteed access within 2.2us, well within the 10us limit

PSM: 
process(CLK33,RESETn)
type TSTATE  is (IDLE,PCIWRITE,PCIREAD,RTACC,RTRD1,RTRD2,RTWR1,RTDONE);
type TMSTATE is (IDLE,PCIWRITE,PCIREAD);
variable STATE   : TSTATE;
variable MSTATE  : TMSTATE;
variable COUNT   : std_logic_vector(3 downto 0);
begin
  if RESETn='0' then
    STATE        := IDLE;
    SSADDR       <= ( others => '0');
    SSDATA       <= ( others => '0');
    SSREAD       <= '0';
    SSWRITE      <= "0000";
    PCIBUSY      <= '0';          
    RD_BE_RDY    <= '0';
    WR_BE_RDY    <= '0';
    PCIACTIVE    <= '0';
    PCIREGACTIVE <= '0';
    PCIACCCNT    <= "000000";
    PCILIMIT     <= '0';
    COREGNT33    <= '0';
    COREWAIT33   <= '0';
    COUNT        := "0000";
    COREDIN      <= ( others => '0');
    RDDELAY      <= "00";
    MSTATE       := IDLE; 
    WNOW         <= "00";  
    PCIREGACTIVE <= '0';
    WDELAY       <= "0000";
    CPURADDR     <= "000";
    CPUDIN       <= ( others => '0');
  elsif CLK33'event and CLK33='1' then
    SSREAD     <= '0';
    SSWRITE    <= "0000";
    RD_BE_RDY  <= '0';
    WR_BE_RDY  <= '0';
    if PCIACCCNT="111111" then
      PCILIMIT <= '1';
    end if;
    PCIACCCNT  <= PCIACCCNT + 1;
    if DP_START='1' and PCIRAMACC='1' then
      PCIACTIVE <= '1';
    end if;
    case STATE is
      when IDLE     => PCIBUSY    <= '0';
                       PCIACCCNT  <= "000000";
                       PCILIMIT   <= '0';
                       COREWAIT33 <= '1';
                       if COREREQ33='1' then
                         STATE := RTACC;
                         COREGNT33 <= '1';
                       elsif PCIACTIVE='1' and RD_CYC='1' then
                         STATE := PCIREAD;
                       elsif PCIACTIVE='1' and WR_CYC='1' then
                         STATE := PCIWRITE;
                       end if;
                       RDDELAY <= "00";
      when PCIWRITE => SSADDR    <= PCIADDR(16 downto 2);
                       SSWRITE   <= WR_BE_NOW;
                       SSDATA    <= PCIDOUT;
                       WR_BE_RDY <= '1';
                       if PCILIMIT='1' then
                         PCIBUSY   <= '1';
                         WR_BE_RDY <= '0';
                       end if;
                       if DP_DONE='1' then
                         WR_BE_RDY <= '0';
                         PCIACTIVE <= '0';
                         STATE     := IDLE;
                       end if;
      when PCIREAD =>  SSADDR  <= PCIADDR(16 downto 2);
                       SSREAD  <= '1';
                       if RDDELAY/="11" then
                         RDDELAY <= RDDELAY + 1;
                       end if;
                       if RDDELAY="10" then
                         RD_BE_RDY <= '1';
                       end if;           
                       if RD_BE_NOW='1' then
                         RDDELAY <= "01";
                       end if;          
                       if PCILIMIT='1' then
                         PCIBUSY   <= '1';
                       end if;
                       if DP_DONE='1'then
                         PCIACTIVE <= '0';
                         STATE  := IDLE;
                       end if;
      when RTACC =>    PCIBUSY <= '1';
                       SSADDR  <= COREADDR(15 downto 1);
                       if CORERD33='1' then
                         STATE := RTRD1;
                         COREGNT33 <= '0';
                       elsif COREWR33='1' then
                         STATE := RTWR1;
                         COREGNT33 <= '0';
                       elsif COREREQ33='0' then
                         STATE := IDLE;
                         COREGNT33 <= '0';
                       end if;
      when RTRD1 =>    STATE := RTRD2;
                       SSREAD <= '1';
      when RTRD2 =>    STATE := RTDONE;
      when RTWR1 =>    STATE := RTDONE;
                       SSDATA     <= COREDOUT & COREDOUT;
                       SSWRITE    <= COREADDR(0) & COREADDR(0) & not COREADDR(0) & not COREADDR(0); 
                       COREWAIT33 <= '0';
      when RTDONE =>   if CORERD33='1' then
                         if COREADDR(0)='0' then
                           COREDIN <= SSDIN_NEG(15 downto 0);
                         else
                           COREDIN <= SSDIN_NEG(31 downto 16);
                         end if;
                       end if;
                       COREWAIT33 <= '0';
                       if COREWR33='0' and CORERD33='0' then
                         STATE := RTACC;
                       end if;
    end case;
    
    -- This handles the PCI access to the 1553B Register
    --  Write cycles data and address are latches then a 3 clock cycle @33MHz write pulse
    --  generated, this must be sampled active by the 16MHz clock in the BC core. The
    --  BC core registers CPUWRn hence there is no metastability problem. 
    --  Read cycles are simple, the BC core, there are no clock dependencies at all, the
    --  address input simply selects what data is multiplexed out. Ready is deactivated
    --  for a clock cycle to allow the address to data delay 
    
    if DP_START='1' and PCIREGACC='1' then
      PCIREGACTIVE <= '1';
    end if;
    WDELAY  <= WDELAY(2 downto 0) & '0';
    case MSTATE is
      when IDLE     => if PCIREGACTIVE='1' and RD_CYC='1' and DP_DONE='0' then
                         MSTATE := PCIREAD;
                         RD_BE_RDY <= '1';
                         CPURADDR  <= PCIADDR(4 downto 2);
                       elsif PCIREGACTIVE='1' and WR_CYC='1' and DP_DONE='0' then
                         MSTATE := PCIWRITE;
                         WR_BE_RDY <= '1';
                         CPURADDR  <= PCIADDR(4 downto 2);
                       end if;
      when PCIREAD  => RD_BE_RDY <= not RD_BE_NOW;
                       CPURADDR <= PCIADDR(4 downto 2);
      when PCIWRITE => WR_BE_RDY <= '0';
                       if WR_BE_NOW/="0000" then
                         WDELAY   <= "0111";
                         WNOW     <= WR_BE_NOW(1 downto 0);
                         CPURADDR <= PCIADDR(4 downto 2);
                       end if;
                       if WDELAY="1000" then
                         MSTATE := IDLE;
                       end if;
    end case;
    if DP_DONE='1' then
      PCIACTIVE    <= '0';
      WR_BE_RDY    <= '0';
      RD_BE_RDY    <= '0';
      PCIREGACTIVE <= '0';
      MSTATE       := IDLE;
    end if;
    if WR_BE_NOW(1)='1' then
      CPUDIN(15 downto 8) <= PCIDOUT(15 downto 8);
    end if;
    if WR_BE_NOW(0)='1' then
      CPUDIN(7 downto 0)  <= PCIDOUT(7 downto 0);
    end if;
  end if;
end process; 


CPUWRITE(0) <= WDELAY(3) and WNOW(0);
CPUWRITE(1) <= WDELAY(3) and WNOW(1);
  

-- OEN to RAM is on -ve clock edges, and the data is sampled on -ve clock edge

process(CLK33)
begin
  if CLK33'event and CLK33='0' then
    SSOEN_NEG <= not SSREAD;
    if SSOEN_NEG='0' then
       SSDIN_NEG <= RAMDATA;
    end if;
  end if;
end process;

-- Route Signals to the SSRAM
SSWRITEANY <= (SSWRITE(3) or SSWRITE(2) or SSWRITE(1) or SSWRITE(0));
RAMADDR    <= SSADDR(14 downto 0);
RAMCSN     <= not (SSREAD or SSWRITEANY);
RAMRDN     <= not SSREAD;
RAMWRN     <= not SSWRITE;
RAMADSN    <= not (SSREAD or SSWRITEANY);
RAMOEN     <= SSOEN_NEG; 
RAMDATA    <= SSDATA when SSWRITEANY='1' else ( others => 'Z');
 
-- PCI read data MUX 
 
process(CPUDOUT,SSDIN_NEG,PCIRAMACC,ZERO)
begin
  if PCIRAMACC='1' then
    PCIDIN <= SSDIN_NEG;
  else
    PCIDIN <= ZERO(31 downto 16) & CPUDOUT;
  end if;
end process;



end RTL;