-- *********************************************************************/ 
-- Copyright 2002 Actel Corporation.  All rights reserved.
-- IP Solutions Group
--  
-- File:  CMDLEGPCI.vhd
--     
-- Description: 1553B RT Development Board
--              Command Word Legallity Checker 
--
-- Rev: 1.0 01Nov02 IPB  : Production Release
-- Rev: 2.1 22Aug03 IPB  : Production 2.1
--
-- Notes:
--
-- *********************************************************************/ 

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

entity CWLEGALPCI is
  port ( CWVAL   : in  std_logic_vector(11 downto 0);
         CMDOKAY : out std_logic;
         TXEN    : in  std_logic_vector(31 downto 0); 
         RXEN    : in  std_logic_vector(31 downto 0) 
        );
end CWLEGALPCI;


architecture RTL of CWLEGALPCI is

------------------------------------------------------------------------------


signal BROADCAST : std_logic;
signal ISMCODE   : std_logic;
signal TX        : std_logic;
signal SA        : std_logic_vector(4 downto 0);
signal WCMC      : std_logic_vector(4 downto 0);
 
 
function to_logic( x: BOOLEAN) return std_logic is
 variable y : std_logic;
 begin
  if x then y:='1';
       else y:='0';
  end if;
  return(y);
end to_logic;


begin

-- Decode incoming Value
BROADCAST <= CWVAL(11); 
TX        <= CWVAL(10); 
SA        <= CWVAL(9 downto 5);
WCMC      <= CWVAL(4 downto 0);
ISMCODE   <= '1' when ( SA="00000" or SA="11111") else '0';


-- This process decodes the Command Word and set CMDOKAY for legal command words

PLEGAL:
process(BROADCAST,TX,SA,WCMC,ISMCODE,TXEN,RXEN)
 variable OK     : std_logic;
 variable MUXSEL : std_logic_vector(5 downto 0);
 variable SAINT  : INTEGER range 0 to 31;
 begin
    
    if (ISMCODE='0') then
        -- Data Transfers
        MUXSEL := TX & SA;
        OK := '0';
        ------------------------------------------------------------------------
        -- This case statement legalizes Data transfers to certain sub addresses
        SAINT := conv_integer(SA);
        if TX='1' then
          OK := TXEN(SAINT);
        else
          OK := RXEN(SAINT);
        end if;  

        ------------------------------------------------------------------------
        -- Broadcast transmits are not allowed, overrides above case statement
        if BROADCAST='1' and TX='1' then
          OK := '0';  -- Broadcast Transmit is not allowed
        end if;
    
     else
        ------------------------------------------------------------------------
        -- This case statement legalizes Mode Codes
        MUXSEL := TX & WCMC;
        OK := '1';
        case MUXSEL is
          when "100000" => --Dynamic Bus Control
                           OK := '0';        -- since we cant do it we messaage error
          when "100001" => --Synchronise
          when "100010" => --Transmit Status Word
                           OK := not BROADCAST;
          when "100011" => --Initiate Self Test, we set this because we provide BIT word
                           OK := '1';
          when "100100" => --Transmitter ShutDown
          when "100101" => --Override Transmitter Shutdown
          when "100110" => --Inhibit terminal flag
          when "100111" => --Override inhibit terminal flag
          when "101000" => --Reset Remote Terminal
          when "110000" => --Transmit Vector Word
                           OK := not BROADCAST;
          when "010001" => --Synchronise with data
          when "110010" => --Transmit last command
                           OK := not BROADCAST;
          when "110011" => --Transmit bit word
                           OK := not BROADCAST;
          when "010100" => --Selected Transmitter Shutdown
                           OK := '0';
          when "010101" => --Override Selected Transmitter Shutdown
                           OK := '0';
          when others   => --All other commands all illegal
                           OK := '0';
        end case;
     end if;
     CMDOKAY <= OK;
end process;


end RTL;