-- ********************************************************************/ 
-- Copyright 2003 Actel Corporation.  All rights reserved.
-- IP Solutions Group
--  
-- File:  pci1553rt.vhd
--     
-- Description: 1553B RT Development Board
--              PCI target to 1553B RT   
--
-- Rev: 1.0 01Nov02 IPB  : Initial Release
-- Rev: 1.1 18Aug03 IPB  : Updated Due fix Synplicity problem
-- Rev: 2.0 22Aug02 IPB  : Production 2.0
-- Rev: 2.1 20Aug03 IPB  : Production 2.1
--
-- Notes:
--
-- *********************************************************************/ 

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

library Core1553B;
use Core1553B.rtcomps.all;

entity PCI1553BRT is
   port    ( -- The PCI Bus Signals
             PCICLK     : in     std_logic;
             PCIRSTN    : in     std_logic;
             AD         : inout  std_logic_vector(31 downto 0);
             CBE        : in     std_logic_vector(3 downto 0); 
             DEVSELN    : out    std_logic;  
             FRAMEN     : in     std_logic;   
             IDSEL      : in     std_logic;   
             INTAN      : out    std_logic;
             IRDYN      : in     std_logic;   
             PAR        : inout  std_logic;  
             PERRN      : out    std_logic;  
             SERRN      : out    std_logic;  
             STOPN      : out    std_logic;  
             TRDYN      : out    std_logic;  
   
             -- 1553B Clock and Tranceiver
             CLK1553    : in  std_logic;
             RESETOUT   : out std_logic;
             RTADDR     : in  std_logic_vector(4 downto 0);
             RTADDRP    : in  std_logic;
             BUSAINEN   : out std_logic;
             BUSAINP    : in  std_logic;
             BUSAINN    : in  std_logic;
             BUSBINEN   : out std_logic;
             BUSBINP    : in  std_logic;
             BUSBINN    : in  std_logic;
             BUSAOUTIN  : out std_logic;
             BUSAOUTP   : out std_logic;
             BUSAOUTN   : out std_logic;
             BUSBOUTIN  : out std_logic;
             BUSBOUTP   : out std_logic;
             BUSBOUTN   : out std_logic;

             -- SSRAM Interface
             RAMADDR    : out   std_logic_vector(11 downto 0);           
             RAMDATA    : inout std_logic_vector(31 downto 0);           
             RAMCSN     : out   std_logic;           
             RAMWRN     : out   std_logic_vector(3 downto 0);           
             RAMRDN     : out   std_logic;           
             RAMADSN    : out   std_logic;           
             RAMOEN     : out   std_logic;
                          
             -- LEDS Etc
             LEDS       : out std_logic_vector(3 downto 0);
             PROBE      : out std_logic_vector(3 downto 0)
           ); 
end PCI1553BRT;   
      
   
architecture RTL of PCI1553BRT is

constant VERSION : std_logic_vector(7 downto 0) := "00000010";

component PCITARG32  
   generic (MHZ_66          : INTEGER RANGE 0 TO 1  := 1;
            MADDR_WIDTH     : INTEGER RANGE 4 TO 31 := 22;
            BAR1_ENABLE     : INTEGER RANGE 0 TO 1  := 1;
            BAR1_IO_MEMORY  : INTEGER RANGE 0 TO 1  := 0; 
            BAR1_ADDR_WIDTH : INTEGER RANGE 2 TO 31 := 8; 
            BAR1_PREFETCH   : INTEGER RANGE 0 TO 1  := 0; 
            HOT_SWAP_ENABLE : INTEGER RANGE 0 TO 1  := 0); 
   port( 
        CLK           : in    std_logic;   
        RST           : in    std_logic;   
        AD            : inout std_logic_vector(31 downto 0); 
        CBE           : in    std_logic_vector(3 downto 0);  
        DEVSELN       : out   std_logic; 
        FRAMEN        : in    std_logic; 
        IDSEL         : in    std_logic; 
        INTAN         : out   std_logic;
        IRDYN         : in    std_logic; 
        PAR           : inout std_logic; 
        PERRN         : out   std_logic; 
        SERRN         : out   std_logic; 
        STOPN         : out   std_logic; 
        TRDYN         : out   std_logic; 
        BAR0_MEM_CYC  : out   std_logic;
        BAR1_CYC      : out   std_logic; 
        BE_GNT        : out   std_logic; 
        BE_REQ        : in    std_logic; 
        BUSY          : in    std_logic; 
        CONFIG_CYC    : out   std_logic;
        DP_DONE       : out   std_logic;
        DP_START      : out   std_logic;
        ERROR         : in    std_logic;     
        EXT_INTN      : in    std_logic;
        MEM_ADD       : out   std_logic_vector(MADDR_WIDTH-3 downto 0);  
        MEM_DIN       : in    std_logic_vector(31 downto 0); 
        MEM_DOUT      : out   std_logic_vector(31 downto 0); 
        PIPE_FULL_CNT : in    std_logic_vector(2 downto 0);
        RD_BE_NOW     : out   std_logic;
        RD_BE_RDY     : in    std_logic;  
        RD_CYC        : out   std_logic;
        WR_BE_NOW     : out   std_logic_vector(3 downto 0);
        WR_BE_RDY     : in    std_logic;   
        WR_CYC        : out   std_logic
       );       
end component; 

component CWLEGALPCI 
  port ( CWVAL   : in  std_logic_vector(11 downto 0);
         CMDOKAY : out std_logic;
         TXEN    : in  std_logic_vector(31 downto 0); 
         RXEN    : in  std_logic_vector(31 downto 0) 
        );
end component;


-- PCI Backend Signals

signal BAR0_MEM_CYC   : std_logic;
signal BAR1_CYC       : std_logic; 
signal CONFIG_CYC     : std_logic;
signal DP_DONE        : std_logic;
signal DP_START       : std_logic;
signal ERROR          : std_logic;     
signal EXT_INTN       : std_logic;
signal PCIADDR        : std_logic_vector(14-2 downto 0);  
signal PCIDIN         : std_logic_vector(31 downto 0); 
signal PCIDOUT        : std_logic_vector(31 downto 0); 
signal PIPE_FULL_CNT  : std_logic_vector(2 downto 0);
signal RD_BE_NOW      : std_logic;
signal RD_BE_RDY      : std_logic;  
signal RD_CYC         : std_logic;
signal WR_BE_NOW      : std_logic_vector(3 downto 0);
signal WR_BE_RDY      : std_logic;   
signal WR_CYC         : std_logic;

-- 1553B Signals
signal  RTADDRx       : std_logic_vector(4 downto 0);
signal  RTADDRPx      : std_logic;
signal  RTADERR       : std_logic;
signal  PCPARITY      : std_logic;
signal  CORESREQUEST  : std_logic;
signal  CORERTBUSY    : std_logic; 
signal  CORESSFLAG    : std_logic;
signal  CORETFLAG     : std_logic;
signal  COREVWORD     : std_logic_vector(15 downto 0);
signal  COREINTACK    : std_logic;
signal  CORECLRERROR  : std_logic;
signal  COREBUSY      : std_logic;
signal  CORECSYNC     : std_logic;
signal  CORESSYNC     : std_logic;
signal  COREINT       : std_logic;
signal  COREIVECT     : std_logic_vector(6 downto 0);
signal  COREMFAIL     : std_logic;
signal  COREWRTTSW    : std_logic;
signal  COREWRTCMD    : std_logic;
signal  COREEXTMDATA  : std_logic;
signal  CORESA30LOOP  : std_logic;
signal  COREBCASTEN   : std_logic;
signal  COREINTENBBR  : std_logic;
      
signal  COREREQn      : std_logic;
signal  COREGNTn      : std_logic;
signal  COREWAITn     : std_logic;
signal  COREWRn       : std_logic;
signal  CORERDn       : std_logic;
signal  CORECSn       : std_logic;
signal  CORECEN       : std_logic;
signal  COREDEN       : std_logic;
signal  COREADDR      : std_logic_vector(10 downto 0);
signal  COREOPER      : std_logic_vector( 1 downto 0);
signal  COREDOUT      : std_logic_vector(15 downto 0);

-- 1553B Backend Signals , top level logic
signal  ADDRLAT       : std_logic;
signal  INTLAT        : std_logic;
signal  CMDOK         : std_logic;
signal  CMDVAL        : std_logic_vector(11 downto 0);
signal  CMDADDR       : std_logic_vector(11 downto 0);
signal  CMDINT        : std_logic_vector(11 downto 0);
signal  WORDCNT       : std_logic_vector(4 downto 0);
signal  SUBADDR       : std_logic_vector(4 downto 0);

-- Address Mapper
signal  RTMEMADDR     : std_logic_vector(12 downto 0);
signal  MAPADDR       : std_logic_vector(12 downto 0);
signal  IVECTMC       : std_logic;
signal  IVECTWC       : std_logic_vector(4 downto 0);
signal  IVECTSA       : std_logic_vector(4 downto 0);
signal  IVECTTX       : std_logic;
signal  IVECTBC       : std_logic;
signal  IVECTOK       : std_logic;

-- Clock Domain Synchronisation
signal  CORERD33      : std_logic;
signal  COREWR33      : std_logic;
signal  COREWAIT33    : std_logic;
signal  COREGNT33     : std_logic;
signal  COREINT33     : std_logic;
signal  COREREQ33     : std_logic;
    
-- Memory Interface State Machine
signal  PCIBUSY       : std_logic;
signal  PCIBUSY2      : std_logic;
signal  PCIRAMACC     : std_logic;
signal  PCIREGACC     : std_logic;
signal  PCIACTIVE     : std_logic;
signal  PCILIMIT      : std_logic;
signal  PCIACCCNT     : std_logic_vector(5 downto 0);
signal  RDDELAY       : std_logic_vector(1 downto 0);
  
-- SSRAM Memory Interface
signal  COREDIN       : std_logic_vector(15 downto 0);
signal  SSWRITEANY    : std_logic;
signal  SSREAD        : std_logic;
signal  SSWRITE       : std_logic_vector( 3 downto 0);
signal  SSDATA        : std_logic_vector(31 downto 0);
signal  SSDIN_NEG     : std_logic_vector(31 downto 0);
signal  SSOEN_NEG     : std_logic;
signal  SSADDR        : std_logic_vector(11 downto 0);

signal  PCCWORD       : std_logic_vector(15 downto 0);
signal  PCVWORD       : std_logic_vector(15 downto 0);

-- 1553B Registers
signal  REG0          : std_logic_vector(31 downto 0);
signal  REG1          : std_logic_vector(31 downto 0);
signal  REG2          : std_logic_vector(31 downto 0);
signal  REG3          : std_logic_vector(31 downto 0);
signal  REG4          : std_logic_vector(31 downto 0);
signal  REG5          : std_logic_vector(31 downto 0);
signal  WRREG0        : std_logic;
signal  WRREG1        : std_logic;
signal  WRREG2        : std_logic;
signal  WRREG3        : std_logic;
signal  WRREG4        : std_logic;
signal  WRREG5        : std_logic;
signal  WR            : std_logic_vector(3 downto 0);
signal  AUTOLOOP      : std_logic;
signal  ADDRMAPPER    : std_logic;
signal  USERTADDR     : std_logic;
signal  REGRTADDR     : std_logic_vector(4 downto 0);
signal  REGRTADDRP    : std_logic;
signal  INTPEND       : std_logic;
signal  INTENABLE     : std_logic;
signal  INCCOUNTER    : std_logic;

-- Miscellanous
signal  CLK33         : std_logic;
signal  CLK16         : std_logic;
signal  RESETn        : std_logic;
signal  LED1          : std_logic;
signal  LED2          : std_logic;
signal  LED3          : std_logic;
signal  LED4          : std_logic;

signal  Logic0        : std_logic;
signal  Logic1        : std_logic;

function to_logic( x: BOOLEAN) return std_logic is
 variable y : std_logic;
 begin
  if x then y:='1';
       else y:='0';
  end if;
  return(y);
end to_logic;

     
begin

Logic0   <= '0';
Logic1   <= '1';
  
-------------------------------------------------------------------------------
-- Clocks etc
--

CLK16 <= CLK1553;
CLK33 <= PCICLK;

RESETn <= PCIRSTn; 

LEDS(0) <= COREBUSY;
LEDS(1) <= AUTOLOOP;
LEDS(2) <= COREMFAIL;
LEDS(3) <= RTADERR;

PROBE <= COREINT & CORECSYNC & CORESSYNC & COREBUSY;

-------------------------------------------------------------------------------
-- The PCI Core
--  

UPCI : PCITARG32  
   generic map (MHZ_66          => 0, 
                MADDR_WIDTH     => 15,
                BAR1_ENABLE     => 0, 
                BAR1_IO_MEMORY  => 0, 
                BAR1_ADDR_WIDTH => 2, 
                BAR1_PREFETCH   => 0, 
                HOT_SWAP_ENABLE => 0  
               )
   port map  (  CLK           =>  CLK33,    
                RST           =>  RESETn,    
                AD            =>  AD,      
                CBE           =>  CBE,    
                DEVSELN       =>  DEVSELN, 
                FRAMEN        =>  FRAMEN, 
                IDSEL         =>  IDSEL,  
                INTAN         =>  INTAN,  
                IRDYN         =>  IRDYN,  
                PAR           =>  PAR,    
                PERRN         =>  PERRN,  
                SERRN         =>  SERRN,  
                STOPN         =>  STOPN,  
                TRDYN         =>  TRDYN, 
                BAR0_MEM_CYC  =>  BAR0_MEM_CYC, 
                BAR1_CYC      =>  BAR1_CYC,     
                BE_GNT        =>  Open,       
                BE_REQ        =>  Logic0,       
                BUSY          =>  PCIBUSY2,         
                CONFIG_CYC    =>  Open,   
                DP_DONE       =>  DP_DONE,      
                DP_START      =>  DP_START,     
                ERROR         =>  ERROR,        
                EXT_INTN      =>  EXT_INTN,     
                MEM_ADD       =>  PCIADDR,      
                MEM_DOUT      =>  PCIDOUT,     
                MEM_DIN       =>  PCIDIN,     
                PIPE_FULL_CNT =>  PIPE_FULL_CNT,
                RD_BE_NOW     =>  RD_BE_NOW,    
                RD_BE_RDY     =>  RD_BE_RDY,    
                RD_CYC        =>  RD_CYC,       
                WR_BE_NOW     =>  WR_BE_NOW,    
                WR_BE_RDY     =>  WR_BE_RDY,    
                WR_CYC        =>  WR_CYC       
              );


PCIRAMACC <= BAR0_MEM_CYC and not PCIADDR(12);
PCIREGACC <= BAR0_MEM_CYC and PCIADDR(12);

ERROR    <= '0';
EXT_INTN <= not (INTPEND and INTENABLE); 

PIPE_FULL_CNT <= "000";

PCIBUSY2 <= PCIBUSY and PCIRAMACC;

-------------------------------------------------------------------------------
-- The 1553B Core
--  

U1553B : RT1553B16 
      port map  (  CLK        => CLK16,      
                   RSTINn     => RESETn,     
                   RTADDR     => RTADDRx, 
                   RTADDRP    => RTADDRPx, 
                   RTADERR    => RTADERR,
                   BUSAINEN   => BUSAINEN, 
                   BUSAINP    => BUSAINP, 
                   BUSAINN    => BUSAINN, 
                   BUSBINEN   => BUSBINEN,  
                   BUSBINP    => BUSBINP, 
                   BUSBINN    => BUSBINN, 
                   BUSAOUTIN  => BUSAOUTIN, 
                   BUSAOUTP   => BUSAOUTP, 
                   BUSAOUTN   => BUSAOUTN, 
                   BUSBOUTIN  => BUSBOUTIN, 
                   BUSBOUTP   => BUSBOUTP,  
                   BUSBOUTN   => BUSBOUTN,  
                   USEXTOK    => Logic1,
                   CMDVAL     => CMDVAL,
                   ADDRLAT    => ADDRLAT,
                   INTLAT     => INTLAT,
                   CMDOK      => CMDOK,
                   CMDOKOUT   => Open,
                   WRTTSW     => COREWRTTSW,
                   WRTCMD     => COREWRTCMD,
                   EXTMDATA   => COREEXTMDATA,
                   ASYNCIF    => Logic1,
                   SA30LOOP   => CORESA30LOOP,
                   BCASTEN    => COREBCASTEN,
                   TESTTXTOUT => Logic0,
                   INTENBBR   => COREINTENBBR,
                   SREQUEST   => CORESREQUEST,
                   RTBUSY     => CORERTBUSY,
                   SSFLAG     => CORESSFLAG,
                   TFLAG      => CORETFLAG,
                   VWORD      => COREVWORD,
                   BUSY       => COREBUSY,
                   MSGSTART   => Open,
                   CMDSYNC    => CORECSYNC,
                   SYNCNOW    => CORESSYNC, 
                   BUSRESET   => RESETOUT, 
                   INTOUT     => COREINT,
                   INTACK     => COREINTACK,
                   CLRERR     => CORECLRERROR,
                   INTVECT    => COREIVECT,        
                   MEMFAIL    => COREMFAIL,
                   MEMREQn    => COREREQn,  
                   MEMGNTn    => COREGNTn,  
                   MEMWRn     => COREWRn,   
                   MEMRDn     => CORERDn,   
                   MEMCSn     => CORECSn,   
                   MEMWAITn   => COREWAITn, 
                   MEMCEN     => CORECEN,   
                   MEMDEN     => COREDEN,   
                   MEMADDR    => COREADDR,  
                   MEMOPER    => COREOPER,
                   MEMDOUT    => COREDOUT,  
                   MEMDIN     => COREDIN   
                  );

-------------------------------------------------------------------------------
-- Legallity Checker
--  

ULEG:  CWLEGALPCI port map
       ( CWVAL   =>  CMDVAL, 
         CMDOKAY =>  CMDOK,
         TXEN    =>  REG3,
         RXEN    =>  REG4
        );


-------------------------------------------------------------------------------
-- Compatibility Address Mapper & Interrupt Vector Extender
--  

process(CLK16,RESETn)
 begin
  if RESETn='0' then
    CMDADDR <= ( others => '0');
  elsif CLK16'event and CLK16='1' then
    if ADDRLAT='1' then
      CMDADDR <= CMDVAL;
    end if;
  end if;
end process;    

process(CMDADDR,COREADDR,COREOPER)
variable SA     : std_logic_vector(4 downto 0);
variable WCCW   : std_logic_vector(4 downto 0);
variable WCAD   : std_logic_vector(4 downto 0);
variable MC     : std_logic;
variable BCAST  : std_logic;
variable TX     : std_logic;
variable MSEL   : std_logic_vector( 2 downto 0);
begin
 SA    := CMDADDR(9 downto 5);
 WCCW  := CMDADDR(4 downto 0);
 WCAD  := COREADDR(4 downto 0);
 BCAST := CMDADDR(11);
 TX    := CMDADDR(10);
 MC    := to_logic(CMDADDR(9 downto 5)="00000") or to_logic(CMDADDR(9 downto 5)="11111"); 
 MSEL  := COREOPER & MC;
 case MSEL is         
   when "100"   => MAPADDR <= BCAST & TX & SA & '0' & "00000";  -- Command Data Transfer
   when "101"   => MAPADDR <= BCAST & TX & SA & '0' & WCCW;     -- Command Mode Code 
   when "010"   => MAPADDR <= BCAST & TX & SA & '0' & "00001";  -- TSW Data Transfer
   when "011"   => MAPADDR <= BCAST & TX & SA & '0' & WCCW;     -- TSW Mode Code 
   when "000"   => MAPADDR <= BCAST & TX & SA & '1' & WCAD;     -- Data Data Transfer
   when "001"   => MAPADDR <= BCAST & TX & SA & '1' & WCCW;     -- Data Mode Code Transfer
   when others  => MAPADDR <= ( others => '-');
 end case;
end process;

RTMEMADDR <= MAPADDR when ADDRMAPPER='1' else "00" & COREADDR;

process(CLK16,RESETn)
 begin
  if RESETn='0' then
    IVECTMC <= '0';
    IVECTSA <= "00000";
    IVECTTX <= '0';
    IVECTWC <= "00000";
    IVECTBC <= '0';
  elsif CLK16'event and CLK16='1' then
    if INTLAT='1' then
      IVECTMC <= to_logic(CMDVAL(9 downto 5)="00000") or to_logic(CMDVAL(9 downto 5)="11111");
      IVECTSA <= CMDVAL(9 downto 5);
      IVECTTX <= CMDVAL(10);
      IVECTWC <= CMDVAL(4 downto 0);
      IVECTBC <= CMDVAL(11);
    end if;
  end if;
end process;    

IVECTOK <= COREIVECT(6);

-------------------------------------------------------------------------------
-- Synchronionise the 1553B RT Signals to the 33MHz PCI Clock
-- 

PMETA33:
process(CLK33)
begin
  if CLK33'event and CLK33='1' then
    COREINT33  <= COREINT;
    COREREQ33  <= not COREREQn;
    CORERD33   <= not CORERDn;
    COREWR33   <= not COREWRn;
  end if;
end process;


PMETA16:
process(CLK16)
begin
  if CLK16'event and CLK16='1' then
    COREGNTn   <= not COREGNT33;
    COREWAITn  <= not COREWAIT33;
  end if;
end process;


-------------------------------------------------------------------------------
-- SSRAM Access Arbritartion etc
--  

-- PCI bus is allowed to do a burst that lasts 63 clocks at 33MHz 
-- allowing for max additional 20 clocks per 1553B access means the
-- 1553B is guarenteed access within 2.2us, well within the 10us limit

PSM: 
process(CLK33,RESETn)
type TSTATE is (IDLE,PCIWRITE,PCIREAD,RTACC,RTRD1,RTRD2,RTWR1,RTDONE,IACK,COPY1,COPY2,COPY3);
variable STATE  : TSTATE;
variable COUNT  : std_logic_vector(3 downto 0);
variable SSADDRP1  : std_logic_vector(3 downto 0);
begin
  if RESETn='0' then
    STATE       := IDLE;
    SSADDR      <= ( others => '0');
    SSDATA      <= ( others => '0');
    SSREAD      <= '0';
    SSWRITE     <= "0000";
    PCIBUSY     <= '0';          
    RD_BE_RDY   <= '0';
    WR_BE_RDY   <= '0';
    PCIACTIVE   <= '0';
    PCIACCCNT   <= "000000";
    PCILIMIT    <= '0';
    COREINTACK  <= '0';
    COREGNT33   <= '0';
    COREWAIT33  <= '0';
    COUNT       := "0000";
    COREDIN     <= ( others => '0');
    INCCOUNTER  <= '0';
    RDDELAY     <= "00";
  elsif CLK33'event and CLK33='1' then
    SSREAD     <= '0';
    SSWRITE    <= "0000";
    RD_BE_RDY  <= '0';
    WR_BE_RDY  <= '0';
    INCCOUNTER <= '0';
    if PCIACCCNT="111111" then
      PCILIMIT <= '1';
    end if;
    PCIACCCNT  <= PCIACCCNT + 1;
    if DP_START='1' and PCIRAMACC='1' then
      PCIACTIVE <= '1';
    end if;
    if COREINTACK='1' and COREINT33='0' then
      COREINTACK <= '0';
    end if;
    case STATE is
      when IDLE     => PCIBUSY    <= '0';
                       PCIACCCNT  <= "000000";
                       PCILIMIT   <= '0';
                       COREWAIT33 <= '1';
                       if COREREQ33='1' then
                         STATE := RTACC;
                         COREGNT33 <= '1';
                       elsif COREINT33='1' then
                         STATE := IACK;
                       elsif PCIACTIVE='1' and RD_CYC='1' then
                         STATE := PCIREAD;
                       elsif PCIACTIVE='1' and WR_CYC='1' then
                         STATE := PCIWRITE;
                       end if;
                       RDDELAY <= "00";
      when PCIWRITE => SSADDR  <= PCIADDR(11 downto 0);
                       for i in 0 to 3 loop
                         SSWRITE(i) <= WR_BE_NOW(i);
                       end loop;
                       SSDATA    <= PCIDOUT;
                       WR_BE_RDY <= '1';
                       if PCILIMIT='1' then
                         PCIBUSY   <= '1';
                         WR_BE_RDY <= '0';
                       end if;
                       if DP_DONE='1' then
                         WR_BE_RDY <= '0';
                         PCIACTIVE <= '0';
                         STATE     := IDLE;
                       end if;
      when PCIREAD =>  SSADDR  <= PCIADDR(11 downto 0);
                       SSREAD  <= '1';
                       if RDDELAY/="11" then
                         RDDELAY <= RDDELAY + 1;
                       end if;
                       if RDDELAY="10" then
                         RD_BE_RDY <= '1';
                       end if;           
                       if RD_BE_NOW='1' then
                         RDDELAY <= "01";
                       end if;          
                       if PCILIMIT='1' then
                         PCIBUSY   <= '1';
                       end if;
                       if DP_DONE='1'then
                         PCIACTIVE <= '0';
                         STATE  := IDLE;
                       end if;
      when RTACC =>    PCIBUSY <= '1';
                       SSADDR  <= RTMEMADDR(12 downto 1);
                       if CORERD33='1' then
                         STATE := RTRD1;
                         COREGNT33 <= '0';
                       elsif COREWR33='1' then
                         STATE := RTWR1;
                         COREGNT33 <= '0';
                       elsif COREREQ33='0' then
                         STATE := IDLE;
                         COREGNT33 <= '0';
                       end if;
      when RTRD1 =>    STATE := RTRD2;
                       SSREAD <= '1';
      when RTRD2 =>    STATE := RTDONE;
      when RTWR1 =>    STATE := RTDONE;
                       SSDATA     <= COREDOUT & COREDOUT;
                       SSWRITE    <= RTMEMADDR(0) & RTMEMADDR(0) & not RTMEMADDR(0) & not RTMEMADDR(0); 
                       COREWAIT33 <= '0';
      when RTDONE =>   if CORERD33='1' then
                         if RTMEMADDR(0)='0' then
                           COREDIN <= SSDIN_NEG(15 downto 0);
                         else
                           COREDIN <= SSDIN_NEG(31 downto 16);
                         end if;
                       end if;
                       COREWAIT33 <= '0';
                       if COREWR33='0' and CORERD33='0' then
                         STATE := RTACC;
                       end if;
      when IACK   =>   COREINTACK <= '1';
                       INCCOUNTER <= IVECTOK;
                       if IVECTMC='0' and AUTOLOOP='1' and IVECTOK='1' then
                         STATE := COPY1;
                       else
                         STATE := IDLE;
                       end if;
                       if ADDRMAPPER='0' then
                         SSADDR <= "000" & IVECTSA & "1111";
                       else
                         SSADDR <= IVECTBC & '0' & IVECTSA & '1' & "1111";
                       end if;
                       COUNT := IVECTWC(4 downto 1);
      when COPY1 =>    PCIBUSY  <= '1';
                       SSADDRP1 := SSADDR(3 downto 0) + 1;
                       SSREAD <= '1';
                       STATE  := COPY2;
                       if ADDRMAPPER='0' then
                         SSADDR <= SSADDR(11 downto 10) & '0' & SSADDR(8 downto 4) & SSADDRP1;
                       else
                         SSADDR <= SSADDR(11 downto 11) & '0' & SSADDR(9 downto 4) & SSADDRP1;
                       end if;
      when COPY2 =>    COUNT := COUNT -1;
                       STATE := COPY3;
      when COPY3 =>    SSWRITE <= "1111";
                       SSDATA  <= SSDIN_NEG;
                       if ADDRMAPPER='0' then
                         SSADDR <= SSADDR(11 downto 10) & '1' & SSADDR(8 downto 0);
                       else
                         SSADDR <= SSADDR(11 downto 11) & '1' & SSADDR(9 downto 0);
                       end if;
                       if COUNT="0000" then
                         STATE := IDLE;
                       else
                         STATE := COPY1;
                       end if;
    end case;
    if PCIREGACC='1' then
      RD_BE_RDY <= '1';
      WR_BE_RDY <= '1';
    end if;
  end if;
end process; 
  

-- OEN to RAM is on -ve clock edges, and the data is sampled on -ve clock edge

process(CLK33)
begin
  if CLK33'event and CLK33='0' then
    SSOEN_NEG <= not SSREAD;
    if SSOEN_NEG='0' then
       SSDIN_NEG <= RAMDATA;
    end if;
  end if;
end process;

-- Route Signals to the SSRAM
SSWRITEANY <= (SSWRITE(3) or SSWRITE(2) or SSWRITE(1) or SSWRITE(0));
RAMADDR    <= SSADDR(11 downto 0);
RAMCSN     <= not (SSREAD or SSWRITEANY);
RAMRDN     <= not SSREAD;
RAMWRN     <= not SSWRITE;
RAMADSN    <= not (SSREAD or SSWRITEANY);
RAMOEN     <= SSOEN_NEG; 
RAMDATA    <= SSDATA when SSWRITEANY='1' else ( others => 'Z');
 
-- PCI read data MUX 
 
process(REG0,REG1,REG2,REG3,REG4,REG5,SSDIN_NEG,PCIADDR,PCIRAMACC)
variable mux : std_logic_vector(2 downto 0);
begin
  mux(2) := PCIADDR(2) or PCIRAMACC;
  mux(1) := PCIADDR(1) or PCIRAMACC;
  mux(0) := PCIADDR(0) or PCIRAMACC;
  case mux is
    when "000"   => PCIDIN <= REG0;
    when "001"   => PCIDIN <= REG1;
    when "010"   => PCIDIN <= REG2;
    when "011"   => PCIDIN <= REG3;
    when "100"   => PCIDIN <= REG4;
    when "101"   => PCIDIN <= REG5;
    when "111"   => PCIDIN <= SSDIN_NEG;
    when others  => PCIDIN <= ( others => '-');
  end case;
end process;


----------------------------------------------------------------------------- 
-- Control Registers
-- Implements the Six 1553B Control registers as per the spec
--
  
WRREG0 <= PCIREGACC and to_logic(PCIADDR(2 downto 0)="000");
WRREG1 <= PCIREGACC and to_logic(PCIADDR(2 downto 0)="001"); 
WRREG2 <= PCIREGACC and to_logic(PCIADDR(2 downto 0)="010");
WRREG3 <= PCIREGACC and to_logic(PCIADDR(2 downto 0)="011");
WRREG4 <= PCIREGACC and to_logic(PCIADDR(2 downto 0)="100");
WRREG5 <= PCIREGACC and to_logic(PCIADDR(2 downto 0)="101");
WR     <= WR_BE_NOW;

PREG: 
process(CLK33,RESETn)
begin
 if RESETn='0' then
   AUTOLOOP     <= '1';
   CORESA30LOOP <= '0';
   ADDRMAPPER   <= '0';
   COREEXTMDATA <= '0';
   COREWRTCMD   <= '0';
   COREWRTTSW   <= '1';                 
   COREBCASTEN  <= '1';
   CORECLRERROR <= '0';
   USERTADDR    <= '0';
   REGRTADDR    <= "00000";
   REGRTADDRP   <= '0';
   CORESREQUEST <= '0';
   CORESSFLAG   <= '0';
   CORERTBUSY   <= '0';
   CORETFLAG    <= '0';
   INTPEND      <= '0';
   INTENABLE    <= '0';
   COREINTENBBR <= '0';
   COREVWORD    <= ( others => '0');
   REG3         <= ( others => '1'); 
   REG4         <= ( others => '1'); 
   REG5         <= ( others => '0'); 
 elsif CLK33'event and CLK33='1' then
   if COREINT33='1' then
     INTPEND <= '1';
   end if;
   if INCCOUNTER='1' then
     REG5 <= REG5 + 1;
   end if;
   if WRREG0='1' and WR(2)='1' then
     AUTOLOOP     <= PCIDOUT(17);
     ADDRMAPPER   <= PCIDOUT(16);
   END IF;
   if WRREG0='1' and WR(1)='1' then
     COREINTENBBR <= PCIDOUT(14);
     CORESA30LOOP <= PCIDOUT(13);
     COREEXTMDATA <= PCIDOUT(12);
     COREWRTCMD   <= PCIDOUT(11);
     COREWRTTSW   <= PCIDOUT(10);                 
     COREBCASTEN  <= PCIDOUT(9);
     CORECLRERROR <= PCIDOUT(8);
   end if;
   if WRREG0='1' and WR(0)='1' then
     USERTADDR    <= PCIDOUT(6);
     REGRTADDRP   <= PCIDOUT(5);
     REGRTADDR    <= PCIDOUT(4 downto 0);
   end if;
   if WRREG1='1' and WR(0)='1' then
     CORESREQUEST <= PCIDOUT(3);
     CORESSFLAG   <= PCIDOUT(2);
     CORERTBUSY   <= PCIDOUT(1);
     CORETFLAG    <= PCIDOUT(0);
   end if;
   if WRREG2='1' and WR(3)='1' then
     if PCIDOUT(31)='1' then
       INTPEND <= '0';
     end if;
     INTENABLE  <= PCIDOUT(30);
   end if;
   if WRREG2='1' and WR(1)='1' then
     COREVWORD(15 downto 8) <= PCIDOUT(15 downto 8);
   end if;
   if WRREG2='1' and WR(0)='1' then
     COREVWORD(7 downto 0)  <= PCIDOUT(7 downto 0);
   end if;
   for i in 0 to 3 loop
     if WRREG3='1' and WR(i)='1' then
       REG3(i*8+7 downto i*8)  <= PCIDOUT(i*8+7 downto i*8);
     end if;
     if WRREG4='1' and WR(i)='1' then
       REG4(i*8+7 downto i*8)  <= PCIDOUT(i*8+7 downto i*8);
     end if;
     if WRREG5='1' and WR(i)='1' then
       REG5(i*8+7 downto i*8)  <= PCIDOUT(i*8+7 downto i*8);
     end if;
   end loop;
 end if;
end process;
      
RTADDRx  <= REGRTADDR  when USERTADDR='1' else RTADDR;
RTADDRPx <= REGRTADDRP when USERTADDR='1' else RTADDRP;

REG0 <= VERSION & "00000" & COREMFAIL & AUTOLOOP  & ADDRMAPPER 
        & '0' &  COREINTENBBR & CORESA30LOOP & COREEXTMDATA 
        &  COREWRTCMD & COREWRTTSW & COREBCASTEN & CORECLRERROR 
        & RTADERR & USERTADDR & REGRTADDRP & REGRTADDR;
    
REG1 <= "0000000000000000000000000000"
        & CORESREQUEST & CORESSFLAG & CORERTBUSY & CORETFLAG;       
       
REG2 <= INTPEND & INTENABLE  
        & IVECTOK & IVECTBC & IVECTMC & IVECTTX & IVECTSA & IVECTWC
        & COREVWORD;    


end RTL;